#Chapter 6 R script

#reading in data
hs <- read.table("hs.dat", header=FALSE)
names(hs) <- c("school", "female", "age", "month", 
               "visperc", "cubes", "lozenges", 
               "parcomp", "sencomp", "wordmean", 
               "addition", "countdot", "sccaps")

#rescaling indicator variables with widely discrepant scale from others
hs$addition <- hs$addition/4
hs$countdot <- hs$countdot/4
hs$sccaps <- hs$sccaps/4

#replacing numeric codes for school with with school names
library(plyr)
hs$school <- as.character(hs$school)
hs$school <- revalue(hs$school, c("0"="Pasteur","1"="Grant White"))

#loading lavaan and semTools
library(lavaan)
library(semTools)

#testing total invariance
mg.1 <- '#measurement model
         visual =~ visperc + cubes + lozenges + sccaps
         verbal =~ parcomp + sencomp + wordmean
         speed =~ addition + countdot + sccaps
        '

fit.total <- cfa(mg.1, data=hs, meanstructure=TRUE, std.lv=TRUE, 
                 group="school",
                 group.equal=c("loadings","intercepts","residuals",
                               "lv.variances","lv.covariances","means"))
summary(fit.total, fit.measures=TRUE)


#evaluting level of invariance

#specifying minimally identified model:
##single indicator per factor with intercepts/loadings equal over groups
##standardized factors in Grant White

mg.2 <- '#loading of first indicator per factor set equal over groups
          visual =~ NA*visperc + c(a,a)*visperc + cubes + lozenges + sccaps
          verbal =~ NA*parcomp + c(b,b)*parcomp + sencomp + wordmean
          speed =~ NA*addition + c(c,c)*addition + countdot + sccaps

          #intercept of first indicator per factor set equal over groups
          visperc ~ c(d,d)*1
          parcomp ~ c(e,e)*1
          addition ~ c(f,f)*1

          #standardizing factor variances in Grant White only
          visual ~~ c(NA,1)*visual
          verbal ~~ c(NA,1)*verbal
          speed ~~ c(NA,1)*speed

          #standardizing factor means in Grant White only
          visual ~ c(NA,0)*1
          verbal ~ c(NA,0)*1
          speed ~ c(NA,0)*1
         '

mi <- measurementInvariance(model=mg.2,data=hs,group="school")
summary(mi$fit.configural)
summary(mi$fit.loadings)
summary(mi$fit.intercepts)

#calculating modification indices for strong invariance model
lavTestScore(mi$fit.intercepts)
param<-parTable(mi$fit.intercepts)
param[c(13,50,33,70),]


#fitting partial invariance model
##removing hard coded equality constraints for minimal identification
##use group.equal instead to set most intercepts/loadings equal over groups
mg.3 <- '#loading of first indicator per factor set equal over groups
          visual =~ NA*visperc + cubes + lozenges + sccaps
          verbal =~ NA*parcomp + sencomp + wordmean
          speed =~ NA*addition + countdot + sccaps

          #standardizing factor variances in Grant White only
          visual ~~ c(NA,1)*visual
          verbal ~~ c(NA,1)*verbal
          speed ~~ c(NA,1)*speed

          #standardizing factor means in Grant White only
          visual ~ c(NA,0)*1
          verbal ~ c(NA,0)*1
          speed ~ c(NA,0)*1
         '

partstr <- cfa(mg.3, data=hs, meanstructure=TRUE, group="school",
               group.equal=c("loadings","intercepts"),
               group.partial=c("lozenges ~1","addition ~1"))
summary(partstr, fit.measures=TRUE, standardized=TRUE)

lavTestLRT(partstr, mi$fit.loadings)
lavTestLRT(mi$fit.intercepts, partstr)

summary(partstr, fit.measures=TRUE, ci=TRUE)

